import time
import random

from StringIO import StringIO

from Testing.ZopeTestCase.threadutils import setNumberOfThreads, QuietThread

from ZServer import asyncore, logger, zhttp_server, zhttp_handler

class ZServerLayer(object):
    """Layer mixin for starting and stopping a ZServer HTTP server on a random
    port. The various class variables may be set prior to setUp() in order to
    influence how the server is run. On tearDown(), the socket will be closed,
    and the server stopped.
    
    Like all of Zope's servers, this relies on an asyncore loop, which will
    be started in a separate thread. As such, running this in parallel in
    multiple threads is likely to be a very bad idea.
    
    In particular, this probably means that this layer cannot be run reliably
    after Testing.ZopeTestCase.utils.startZServer() has been called.
    """
    
    host = '127.0.0.1'
    port = random.choice(range(55000, 55500))
    log = StringIO()
    _threads = 1
    
    @classmethod
    def setUp(cls):
        cls._shutdown = False
        setNumberOfThreads(cls._threads)
        t = QuietThread(target=cls._runner, args=())
        t.setDaemon(1)
        t.start()
        time.sleep(0.1)
    
    @classmethod
    def tearDown(cls):
        cls._shutdown = True
    
    @classmethod
    def _runner(cls):
        """Start a server using asyncore - this runs in a separate thread!
        """
        log = logger.file_logger(cls.log)
        
        # Create ZServer
        server = zhttp_server(ip=cls.host, port=cls.port, logger_object=log)
        handler = zhttp_handler(module='Zope2', uri_base='')
        server.install_handler(handler)
        
        # Poll
        timeout = 30.0
        socket_map = asyncore.socket_map
        while socket_map and not cls._shutdown:
            asyncore.poll(timeout, socket_map)
        
        # Close socket
        server.close()



from Products.PloneTestCase import PloneTestCase as ptc

from niteoweb.windmill import WindmillTestCase

ptc.setupPloneSite()

class TestWM(WindmillTestCase):
    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.login_user()

    def test_foo(self):
        print 'ok'


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestWM))
    return suite



import sys
import logging
import posixpath as url_path

import transaction

from Products.PloneTestCase.PloneTestCase import FunctionalTestCase, portal_owner, default_password
from Products.PloneTestCase.layer import PloneSite as PloneLayer

import windmill

from windmill.bin.admin_lib import configure_global_settings, setup, teardown
from windmill.authoring.unit import WindmillUnitTestCase
from windmill.authoring import WindmillTestClient

from niteoweb.windmill.zserver import ZServerLayer

log = logging.getLogger(__name__)

class WindmillLayer(ZServerLayer):
    """Layer for running windmill. This depends on the ZServer layer, which
    means that it will start up a ZServer instance on a random port.
    See zserver.py for details.
    """
    
    site = 'plone'
    
    windmill_settings = {
        'START_FIREFOX': True,
    }
    
    test_url = "http://%(host)s:%(port)s/index_html"

    @classmethod
    def setUp(cls):
        
        # Start the Zope server with one thread
        host, port = cls.host, cls.port
        cls.test_url = cls.test_url % locals()
        log.debug('Starting ZServer on: %s' % cls.test_url)

        # Configure windmill
        windmill.stdout, windmill.stdin = sys.stdout, sys.stdin
        configure_global_settings()
        windmill.settings['TEST_URL'] = cls.test_url
        windmill.settings.update(getattr(cls, "windmill_settings", {}))
        cls.windmill_shell_objects = setup()
        log.debug('Starting Windmill with settings: %r' % windmill.settings)

        # Configure client
        cls.wm = WindmillTestClient(__name__)
        def open_site(*a, **kw):
            """little hack to always open an url with name of plone site"""
            transaction.commit()
            if 'url' in kw:
                kw['url'] = url_path.join('/' + (kw.get('site', None) or cls.site), kw['url'].lstrip('/'))
            return cls.wm.open(*a, **kw)
        cls.wm.open_site = open_site

    @classmethod
    def tearDown(cls):
        teardown(cls.windmill_shell_objects)


class WindmillPloneLayer(PloneLayer, WindmillLayer):
    """Convenience layer which mixes PloneLayer from PloneTestCase in with
    WindMillLayer. Use this layer to execute a windmill test against a Plone
    site configured using the normal PloneTestCase setup.
    """


class WindmillTestCase(FunctionalTestCase):
    """Convenient test case which makes a Windmill controller availalbe on
    self.wm and adds some convenience methods.
    """
    
    layer = WindmillPloneLayer

    @property
    def wm(self):
        return self.layer.wm

    def login_user(self, username=portal_owner, password=default_password):
        self.wm.open_site(url="/login_form")
        self.wm.waits.forPageLoad(timeout=30000)
        self.wm.type(id="__ac_name", text=username)
        self.wm.type(id="__ac_password", text=password)
        self.wm.click(name="submit")
        self.wm.waits.forPageLoad(timeout=30000)
        self.wm.asserts.assertNode(xpath="//dl[@class='portalMessage info']/dd")

    def add_user(self, username, password, roles=('Manager',)):
        self.portal.acl_users.userFolderAddUser(username, password, roles, [])
        transaction.commit()



import os.path
from setuptools import setup, find_packages

DIR = os.path.dirname(os.path.abspath(__file__))

setup(name='niteoweb.windmill',
      version='0.3.1',
      description="Plone TestCase integration with Windmill testing",
      long_description=open(os.path.join(DIR, 'README.txt')).read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zope plone windmill',
      author='Niteoweb d.o.o.',
      author_email='info@niteoweb.com',
      url='http://www.niteoweb.com/',
      license='GNU GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['niteoweb'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'windmill',
      ],
      entry_points="""""",
      )
